import typing

from rest_framework import serializers

from .base import AdminBaseFieldMixin


class AdminJsonField(AdminBaseFieldMixin, serializers.JSONField):
    json_forms: typing.Optional[dict]

    def __init__(self, **kwargs):
        self.json_forms = kwargs.pop('json_forms', None)
        super().__init__(**kwargs)



import typing

from django.utils.translation import gettext as _
from rest_framework import serializers


class AdminChoiceField(serializers.ChoiceField):
    tag_style: typing.Optional[dict]
    max_width: typing.Optional[int]

    def __init__(self, *args, **kwargs):
        kwargs.pop("child", None)
        self.tag_style = kwargs.pop('tag_style', {})
        self.max_width = kwargs.pop("max_width", None)
        super().__init__(*args, **kwargs)

    def to_representation(self, value):
        if isinstance(value, list):
            return value

        if value in self._choices:
            return value

        return value

    def _translate_value(self, value):
        if isinstance(value, int):
            return value
        return str(_(value))

    def to_internal_value(self, data):
        if data == '' and self.allow_blank:
            return ''

        value = data
        if isinstance(data, dict):
            value = data["value"]

        for key, val in self._choices.items():
            if key == value:
                return value

        self.fail('invalid_choice', input=data)



from custom_admin.controllers.permissions import CheckPermissions, PermissioinType
from custom_admin.controllers.schema_generator import ViewSetSchemaGenerator


def get_schema(router, request) -> dict:
    sections = {}

    for _prefix, viewset, basename in router.registry:

        has_perm = CheckPermissions(request.user).has_perm(viewset, PermissioinType.VIEW)
        if not has_perm:
            continue

        if has_perm:
            generator = ViewSetSchemaGenerator(viewset, router, basename, request)
            sections[basename] = generator.get_viewset_urls()

    return sections



import typing
from dataclasses import asdict, dataclass, field

from custom_admin.api.inlines.interfaces import InlineResultInterface

PLUGINS = {
    'title': {
        'display': True,
    },
    'tooltip': {
        'mode': 'index',
        'intersect': False,
    },
}


@dataclass
class ChartOptions:
    width: typing.Optional[int] = None
    height: typing.Optional[int] = 80

    plugins: typing.Dict[str, dict] = field(default_factory=lambda: PLUGINS)


@dataclass
class GraphDataset:
    # читаемое название
    label: str

    data: typing.List[dict]

    # https://www.chartjs.org/docs/latest/charts/area.html#filling-modes
    fill: str = 'false'

    backgroundColor: str = 'rgb(200, 200, 200)'

    pointBorderColor: typing.Optional[str] = None

    borderColor: str = 'gray'
    borderWidth: int = 1

    pointBorderWidth: int = 1
    tension: float = 0.1


class ChartType:
    LINE = 'line'
    BAR = 'bar'
    BUBBLE = 'bubble'
    DOUGHNUT = 'doughnut'
    PIE = 'pie'
    POLARAREA = 'polararea'
    RADAR = 'radar'
    SCATTER = 'scatter'


@dataclass
class ChartData:
    type: ChartType = ChartType.LINE

    labels: typing.List[str] = field(default_factory=list)
    datasets: typing.List[GraphDataset] = field(default_factory=list)
    options: ChartOptions = field(default_factory=ChartOptions)


@dataclass
@InlineResultInterface.register
class InlineGraphResult:
    messages: typing.List[dict] = field(default_factory=dict)

    charts: typing.List[ChartData] = field(default_factory=list)

    def asdict(self) -> dict:
        return asdict(self)



# Generated by Django 4.2.7 on 2024-04-11 18:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AdminLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('section', models.CharField(max_length=64, verbose_name='Раздел')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Заголовок')),
                ('action_type', models.CharField(choices=[('create', 'Создание'), ('edit', 'Редактирование'), ('action', 'Действие')], max_length=12, verbose_name='Тип действия')),
                ('content', models.JSONField(default=dict, verbose_name='Содержимое действия')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Время совершения действия')),
                ('staff', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='admin_actions', to=settings.AUTH_USER_MODEL, verbose_name='Персонал')),
            ],
            options={
                'verbose_name': 'Лог действия администратора',
                'verbose_name_plural': 'Лог действий администраторов',
            },
        ),
    ]



from django.contrib.auth import get_user_model
from django.urls import reverse
from rest_framework.test import APITestCase

User = get_user_model()


class SchemeTestAPI(APITestCase):
    urls = 'custom_admin.api.tests.urls'

    def test_scheme_api(self):
        user = User.objects.create(username="username", is_staff=True)
        self.client.force_authenticate(user=user)
        response = self.client.get(reverse('get-sections'))
        self.assertEqual(response.status_code, 200, response.content.decode())



import typing
from enum import Enum

from django.db.models import Q
from django.contrib.auth.models import Permission


# All default views: list send-action retrieve update partial_update destroy
class PermissioinType(Enum):
    VIEW = 'view'
    CHANGE = 'change'
    ADD = 'add'
    ACTIONS = 'send_action'


class CheckPermissions:
    user = None

    def __init__(self, user):
        self.user = user

    def _has_perm(self, codename: str):
        if not self.user.is_active:
            return False

        if self.user.is_superuser:
            return True

        has_perm = self.user.user_permissions.filter(codename=codename)
        has_group = self.user.groups.filter(permissions__codename=codename)
        return has_perm.exists() or has_group.exists()

    def has_action_perm(self, viewset, action: str) -> bool:
        codename = f'{self.get_codename(action)}_{viewset.get_view_viewname()}'
        return self._has_perm(codename)

    def has_perm(self, viewset, perm_type: PermissioinType) -> bool:
        codename = f'{perm_type.value}_{viewset.get_view_viewname()}'
        return self._has_perm(codename)

    def has_perm_viewname(self, viewname: str, perm_type: PermissioinType) -> bool:
        codename = f'{perm_type.value}_{viewname}'
        return self._has_perm(codename)

    @classmethod
    def get_codename(cls, action: str) -> str:
        if action in ['list', 'retrieve']:
            return PermissioinType.VIEW.value
        if action in ['update', 'partial_update']:
            return PermissioinType.CHANGE.value
        if action == 'destroy':
            return PermissioinType.DELETE.value
        if action == 'create':
            return PermissioinType.ADD.value

        return action

    @classmethod
    def get_all_permissions(cls, viewset) -> typing.Tuple[str]:
        from custom_admin.api.urls import admin_router as router
        routes = router.get_routes(viewset)

        result = set()
        for route in routes:
            mappings = router.get_method_map(viewset, route.mapping)
            for method, method_name in mappings.items():
                result.add(
                    f'{cls.get_codename(method_name)}_{viewset.get_view_viewname()}'
                )

        return tuple(result)



import asyncio as aio

from asgiref.sync import sync_to_async


# https://github.com/encode/django-rest-framework/discussions/7774
class AsyncMixin:
    """Provides async view compatible support for DRF Views and ViewSets.

    This must be the first inherited class.

        class MyViewSet(AsyncMixin, GenericViewSet):
            pass
    """
    @classmethod
    def as_view(cls, *args, **initkwargs):
        """Make Django process the view as an async view.
        """
        view = super().as_view(*args, **initkwargs)

        async def async_view(*args, **kwargs):
            # wait for the `dispatch` method
            return await view(*args, **kwargs)
        async_view.csrf_exempt = True
        return async_view

    async def dispatch(self, request, *args, **kwargs):
        """Add async support.
        """
        self.args = args
        self.kwargs = kwargs
        request = self.initialize_request(request, *args, **kwargs)
        self.request = request
        self.headers = self.default_response_headers

        try:
            await sync_to_async(self.initial)(
                request, *args, **kwargs)  # MODIFIED HERE

            if request.method.lower() in self.http_method_names:
                handler = getattr(
                    self, request.method.lower(),
                    self.http_method_not_allowed
                )
            else:
                handler = self.http_method_not_allowed

            # accept both async and sync handlers
            # built-in handlers are sync handlers
            if not aio.iscoroutinefunction(handler):  # MODIFIED HERE
                handler = sync_to_async(handler)  # MODIFIED HERE
            response = await handler(request, *args, **kwargs)  # MODIFIED HERE

        except Exception as exc:
            response = self.handle_exception(exc)

        self.response = self.finalize_response(
            request, response, *args, **kwargs)
        return self.response



import json
import typing
from json.decoder import JSONDecodeError
from urllib.parse import unquote

from django_filters import rest_framework
from rest_framework import filters, serializers

from custom_admin.api.filters.base_admin_filter import BaseAdminFilterSet


def get_filter_info(request) -> typing.Optional[dict]:
    if not request.GET or 'filter_info' not in request.GET:
        return

    filter_info = request.GET['filter_info']
    if not filter_info:
        return

    try:
        return json.loads(unquote(filter_info))
    except JSONDecodeError as e:
        raise serializers.ValidationError(f'Error filter decode; {filter_info=} {e=}')


class CustomSearchFilter(filters.SearchFilter):
    def get_search_terms(self, request):
        search = super().get_search_terms(request)

        filter_info = get_filter_info(request)
        if filter_info:
            _search = filter_info.get('search')
            if _search:
                return [_search]

        return search


class CustomOrderingFilter(filters.OrderingFilter):

    def get_ordering(self, request, queryset, view):
        ordering = super().get_ordering(request, queryset, view)

        filter_info = get_filter_info(request)
        if filter_info:
            _ordering = filter_info.get('ordering')
            if _ordering:
                return [_ordering]

        return ordering


class CustomFilterBackend(rest_framework.DjangoFilterBackend):
    filterset_base = BaseAdminFilterSet

    def get_filterset_kwargs(self, request, queryset, view):
        data = request.query_params

        filter_info = get_filter_info(request)
        if filter_info:
            _filters = filter_info.get('filters')
            if _filters:
                data = _filters

        return {
            'data': data,
            'queryset': queryset,
            'request': request,
        }



from rest_framework.fields import DecimalField, IntegerField


class PositiveIntegerField(IntegerField):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.min_value = 0


class PositiveDecimalField(DecimalField):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.min_value = 0.0



import typing

from django.conf import settings
from django.utils.translation import gettext as _
from django_filters.utils import get_model_field

from custom_admin.api.action_functions import ACTION_ATTRIBUTES
from custom_admin.api.serializers.base_serializer import AdminModelSerializer
from custom_admin.api.viewset_info import AdminViewSetInfo
from custom_admin.controllers.filters_schema import (
    ModelFieldException,
    get_filters_class_data,
    get_filters_fields_data,
)
from custom_admin.controllers.permissions import CheckPermissions, PermissioinType


class ViewSetSchemaGenerator:
    viewset = None
    router = None
    basename = None
    request = None
    model = None

    def __init__(self, viewset, router, basename, request):
        self.viewset = viewset
        self.router = router
        self.basename = basename
        self.request = request

        if hasattr(self.viewset, 'get_model'):
            self.model = viewset.get_model()

    def _get_inline_data(self, initkwargs: dict) -> typing.Optional[dict]:
        inline_type = initkwargs.get('inline_type')

        if inline_type is None:
            return

        filterset_fields = {}
        filterset_class = initkwargs.get('filterset_class')
        if filterset_class is not None:
            filterset_fields = get_filters_class_data(None, filterset_class, self.request)

        return {
            'inline_type': inline_type,
            'filterset_fields': filterset_fields,
            'actions': self._get_actions_info(initkwargs.get('actions', [])),
        }

    def _generate_view_data(self, route, lookup) -> typing.Union[str, dict]:
        'Its for spicific method: user-detail or user-list'

        mappings = self.router.get_method_map(self.viewset, route.mapping)

        regex = route.url.format(
            prefix=self.basename,
            lookup=lookup,
            trailing_slash=self.router.trailing_slash
        )

        if not self.basename and regex[:2] == '^/':
            regex = '^' + regex[2:]

        initkwargs = route.initkwargs.copy()
        initkwargs.update({
            'detail': route.detail,
        })

        name = initkwargs.get('name', route.name.format(basename=self.basename))
        name = _(name)

        mappings_info = {}
        for method, method_name in mappings.items():
            has_perm = CheckPermissions(self.request.user).has_action_perm(self.viewset, method_name)
            if has_perm:
                mappings_info[method] = method_name

        url_data = {
            'slug': route.name.format(basename=self.basename),
            'mapping': mappings_info,
            'name': name,
            'detail': route.detail,
        }

        inline_data = self._get_inline_data(initkwargs)
        if inline_data is not None:
            url_data.update(inline_data)

        return name, url_data

    def _get_filters_data(self):
        filters_data = []

        if self.model:
            filters = getattr(self.viewset, 'filterset_fields', None)
            if filters:
                try:
                    filters_data = get_filters_fields_data(self.model, filters)
                except ModelFieldException as e:
                    raise ModelFieldException(f'View {self.viewset.__name__}: {e}')

            filter_class = getattr(self.viewset, 'filter_class', None) or getattr(self.viewset, 'filterset_class', None)
            if filter_class:
                filters_data = get_filters_class_data(self.model, filter_class, self.request)

        return filters_data

    def _get_serializer_info(self):
        serializer = None
        serializer_info = None

        meta = self.viewset.metadata_class()
        if hasattr(self.viewset, 'get_serializer_class') and self.viewset.get_serializer_class():
            serializer = self.viewset.get_serializer_class()()
            try:
                serializer_info = meta.get_serializer_info(serializer)
            except Exception as e:
                raise Exception(f'Error in {self.viewset.__name__}: {e}')

            if not issubclass(serializer.__class__, (AdminModelSerializer,)):
                raise Exception(
                    'Admin serializer {} must be inherited from AdminModelSerializer'.format(
                        serializer.__class__.__name__
                    )
                )

            if 'id' not in serializer_info.keys():
                raise Exception(
                    f'Make sure that {serializer.__class__.__name__} have id field'
                )

        return serializer, serializer_info

    def _get_list_display(self, serializer_info, serializer):
        list_display = getattr(self.viewset, 'list_display', None)
        if list_display:
            if 'id' not in list_display:
                list_display = list(list_display) + ['id']

            for field_display in list_display:
                if field_display not in serializer_info.keys():
                    raise Exception(
                        f'Field "{field_display}" from {self.viewset.__name__}.list_display not presented in {serializer.__class__.__name__}'
                    )
        return list_display

    def _get_search_fields(self):
        result_search_fields = []
        search_fields = getattr(self.viewset, 'search_fields', [])
        if search_fields:
            if self.model:
                for search_field in search_fields:
                    field = get_model_field(self.model, search_field)
                    if field:
                        name = field.verbose_name
                        if name == 'ID':
                            name = f'{_(field.model._meta.verbose_name)} ID'
                        result_search_fields.append(_(name))
                    else:
                        result_search_fields.append(_(search_field))
            else:
                result_search_fields = search_fields

        return result_search_fields

    def _get_actions_info(self, view_actions) -> dict:
        actions = {}

        has_perm = CheckPermissions(self.request.user).has_perm(self.viewset, PermissioinType.ACTIONS)
        if not has_perm:
            return actions

        for action in view_actions:
            name = getattr(action, 'short_description', action.__name__)
            actions[action.__name__] = {'name': name}

            for attr in ACTION_ATTRIBUTES:
                actions[action.__name__][attr] = getattr(action, attr, None)

            form_serializer = getattr(action, 'form_serializer', None)
            if form_serializer:
                meta = self.viewset.metadata_class()
                serializer_info = meta.get_serializer_info(form_serializer())
                actions[action.__name__]['form_serializer'] = serializer_info
        return actions

    def _get_translations_info(self, serializer):
        try:
            from modeltranslation.translator import NotRegistered, translator
        except ModuleNotFoundError:
            return {}

        if not self.model:
            return {}

        result = {}
        try:
            for field in translator.get_options_for_model(self.model).fields:
                result[field] = []
                for lang_slug, lang_translation in settings.LANGUAGES:
                    result[field].append({
                        'slug': f'{field}_{lang_slug}',
                        'lang_slug': lang_slug,
                        'lang_translation': str(lang_translation),
                    })

            from custom_admin.api.serializers import TranslatedModelSerializer
            if not isinstance(serializer, TranslatedModelSerializer):
                raise ModelFieldException(f'Serializer {serializer.__class__.__name__} has translations, but not instance of TranslatedModelSerializer')

        except NotRegistered:
            pass
        return result


    def get_viewset_urls(self) -> dict:
        viewset_urls = {}

        title = str(_(self.viewset.get_view_title()))
        filters_data = self._get_filters_data()
        serializer, serializer_info = self._get_serializer_info()
        list_display = self._get_list_display(serializer_info, serializer)
        search_fields = self._get_search_fields()

        related_inlines = []
        if hasattr(self.viewset, 'get_related_inlines'):
            for related_inline in self.viewset.get_related_inlines():
                has_perm = CheckPermissions(self.request.user).has_perm_viewname(
                    viewname=related_inline.viewset_name,
                    perm_type=PermissioinType.VIEW,
                )
                if has_perm:
                    related_inlines.append(related_inline.asdict())

        meta_data = {
            'serializer': serializer_info,
            'search_fields': search_fields,
            'ordering_fields': getattr(self.viewset, 'ordering_fields', []),
            'related_inlines': related_inlines,
            'field_groups': getattr(serializer.Meta, 'groups', None) if serializer else None,
            'filterset_fields': filters_data,
            'filds_list': list_display,
            'fixed_columns': getattr(self.viewset, 'fixed_columns', []),
            'actions': self._get_actions_info(getattr(self.viewset, "actions", [])),
            'translations': self._get_translations_info(serializer),
        }

        lookup = self.router.get_lookup_regex(self.viewset)
        viewset_urls = {
            'viewname': self.viewset.get_view_viewname(),
            'lookup': lookup,
            'title': str(title),
            'routers': {},
            'hide_in_navigation': self.viewset.hide_in_navigation,
            'icon': self.viewset.icon,
            'meta': meta_data,
        }

        viewset_info: AdminViewSetInfo = self.viewset._info
        viewset_urls.update(viewset_info.asdict())

        routes = self.router.get_routes(self.viewset)
        for route in routes:
            route_name, url_data = self._generate_view_data(route, lookup)
            viewset_urls['routers'][route_name] = url_data

        return viewset_urls



import datetime
import typing
from io import BytesIO

from django.http import HttpResponse
from django.utils.translation import gettext_lazy as _

from custom_admin.api.action_functions import admin_action


def get_list_display(view):
    list_display = getattr(view, 'list_display', None)
    if list_display:
        return list_display

    meta = view.metadata_class()
    serializer_class = view.get_serializer_class()
    serializer_info = meta.get_serializer_info(serializer_class())
    return serializer_info.keys()


def _get_headers(view, serializer, serializer_class):
    meta = view.metadata_class()
    serializer_info = meta.get_serializer_info(serializer_class())
    field_data = {
        field: data.get('label')
        for field, data in serializer_info.items()
    }
    serializer_fields = getattr(serializer, 'get_fields', {})
    headers = []
    for column in get_list_display(view):
        serializer_field = serializer_fields.get(column)
        if serializer_field and serializer_field.label:
            headers.append(serializer_field.label)
        elif column in field_data:
            headers.append(field_data[column])
        else:
            headers.append(column)

    return headers


def base_export_csv(filename: str, columns: typing.List, lines: typing.List):
    f = BytesIO()
    import unicodecsv as csv
    writer = csv.writer(f, dialect=csv.excel, encoding='utf-8')
    writer.writerow(columns)

    for line in lines:
        writer.writerow(line)
    f.seek(0)

    response = HttpResponse(f, content_type='text/csv')
    response['Pragma'] = filename
    response['Content-Disposition'] = f'attachment; filename="{filename}"'
    f.close()
    return response


@admin_action(
    short_description=_("Выгрузить в csv"),
    icon='mdi-application-export',
)
def export_csv_action(view, request, queryset, *args, **kwargs):
    serializer_class = view.get_serializer_class()
    serializer = serializer_class(queryset, many=True, context={'request': request})

    columns = _get_headers(view, serializer, serializer_class)
    filename = f'{view.get_view_viewname()}_{datetime.datetime.now()}.csv'

    lines = []
    for data in serializer.data:
        line_data = []

        for column in get_list_display(view):
            value = data.get(column, None)
            if isinstance(value, dict):
                value = value.get('text', value)

            line_data.append(value)

        lines.append(line_data)

    return base_export_csv(filename, columns, lines)



import datetime

from django.utils.translation import gettext_lazy as _

from custom_admin.api.actions.export_csv_action import base_export_csv
from custom_admin.api.inlines.inline_table import InlineTableResult


def export_csv_inline(view, request, result: InlineTableResult, *args, **kwargs):
    data = result.asdict()
    columns = data.get('columns')
    lines = data.get('data')
    filename = f'{view.__name__}_{datetime.datetime.now()}.csv'
    return base_export_csv(filename, columns, lines)


export_csv_inline.short_description = _("Выгрузить в csv (кодировка UTF-8)")



import typing

from rest_framework import serializers

from .base import AdminBaseFieldMixin


class AdminCharField(AdminBaseFieldMixin, serializers.CharField):
    '''
    Поле wysiwyg используется на фронте для отображения
    визуального редактора текста. Он сохраняет текст в формате html.

    multilined - для отображения текстового поля в несколько строк
    '''

    wysiwyg: bool
    multilined: bool
    tag_style: typing.Optional[dict]

    def __init__(self, **kwargs):
        self.wysiwyg = kwargs.pop('wysiwyg', False)
        self.tag_style = kwargs.pop('tag_style', {})
        self.multilined = kwargs.pop('multilined', False)
        super().__init__(**kwargs)



import json

from django.conf import os, settings
from django.shortcuts import render
from django.template.exceptions import TemplateDoesNotExist
from django.views import View

from custom_admin.utils import get_schema

TEMPLATE_NOT_FOUND = '''
Custom admin cannot find template "{template}".
Make sure settings.TEMPLATES contains 'APP_DIRS': True
'''


class CustomAdminView(View):
    '''
    # Settings in case of h2ttps:
    USE_X_FORWARDED_HOST = True
    SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
    '''

    template = 'custom_admin/admin_index.html'

    def get(self, request):
        custom_admin = getattr(settings, 'CUSTOM_ADMIN', {})

        admin_settings = {
            'title': custom_admin.get('title', 'Admin'),
            'backend_prefix': custom_admin.get('backend_prefix'),
        }

        if not admin_settings.get('backend_prefix'):
            admin_settings['backend_prefix'] = request.build_absolute_uri('/custom_admin/')

        context = {
            'SETTINGS': admin_settings,
            'SETTINGS_JSON': json.dumps(admin_settings),
        }

        try:
            return render(request, self.template, context)
        except TemplateDoesNotExist as e:
            raise TemplateDoesNotExist(TEMPLATE_NOT_FOUND.format(template=self.template)) from e



from django.contrib.auth.models import Permission
from custom_admin.api.views.base_admin_viewset import BaseAdminViewSet
from django.utils.translation import gettext_lazy as _


class PermissionsAdminViewSet(BaseAdminViewSet):
    title = _('Права доступа')
    queryset = Permission.objects.all().select_related('content_type')
    search_fields = ['id', 'name', 'codename']
    filterset_fields = ['content_type']
    list_display = (
        'id',
        'name',
        'codename',
        'content_type',
    )



import functools
import typing

ACTION_ATTRIBUTES = [
    'description',
    'short_description',
    'confirmation_text',
    'icon',
    'variant',
    'base_color',
    'allow_empty_selection',
]

def admin_action(
        description: typing.Optional[str] = None,
        short_description: typing.Optional[str] = None,
        confirmation_text: typing.Optional[str] = None,

        # https://pictogrammers.com/library/mdi/
        icon: typing.Optional[str] = None,

        # elevated, flat, tonal, outlined, text, and plain.
        variant: typing.Optional[str] = None,

        base_color: typing.Optional[str] = None,

        allow_empty_selection: bool = False,

        form_serializer=None,
):
    def decorator(func):
        func.description = description
        func.short_description = short_description
        func.confirmation_text = confirmation_text
        func.icon = icon
        func.variant = variant
        func.base_color = base_color
        func.allow_empty_selection = allow_empty_selection

        if form_serializer:
            from custom_admin.api.serializers import AdminSerializer
            if not issubclass(form_serializer, AdminSerializer):
                raise Exception(f'{func.__name__}.form_serializer must be instance of AdminSerializer')
            func.form_serializer = form_serializer

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            return result
        return wrapper
    return decorator



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import gettext as _
from rest_framework import fields, relations
from rest_framework.exceptions import ValidationError
from rest_framework.fields import empty
from rest_framework.relations import ManyRelatedField, MANY_RELATION_KWARGS

log = logging.getLogger("base-admin-serializer")


class AdminManyRelatedField(ManyRelatedField):
    def run_validation(self, data=empty):
        """
            Фиксит проблему (в serializers.ModelSerializer, когда он строит поле
             django.db.models.models.ManyToManyField), когда ставишь потом удаляешь значение,
             необоснованно вызывается исключение "Это поле не может быть пустым"
             при data = [] такого не происходит
        """
        if data is None:
            data = []
        return super().run_validation(data)


class AdminPrimaryKeyRelatedField(relations.PrimaryKeyRelatedField):
    instance = None
    filter_queryset = None
    update_only = None
    create_only = None

    def __init__(self, **kwargs):
        self.filter_queryset = kwargs.pop('filter_queryset', None)
        self.update_only = kwargs.pop('update_only', False)
        self.create_only = kwargs.pop('create_only', False)
        super().__init__(**kwargs)

    @classmethod
    def many_init(cls, *args, **kwargs):
        list_kwargs = {'child_relation': cls(*args, **kwargs)}
        for key in kwargs:
            if key in MANY_RELATION_KWARGS:
                list_kwargs[key] = kwargs[key]

        return AdminManyRelatedField(**list_kwargs)

    def get_attribute(self, instance):
        if self.use_pk_only_optimization() and self.source_attrs:
            try:
                field_name = self.source_attrs[-1]
                attribute_instance = fields.get_attribute(instance, self.source_attrs[:-1])
                value = attribute_instance.serializable_value(field_name)

                if fields.is_simple_callable(value):
                    return value()
                else:
                    return getattr(instance, field_name, None)

            except AttributeError:
                pass

    def to_representation(self, value):
        if self.pk_field is not None:
            return self.pk_field.to_representation(value.pk)

        def get_str_value(obj):
            if obj:
                try:
                    return str(obj)
                except TypeError as e:
                    log.error(
                        'to_representation error: cant get str from %s with id:%s return None',
                        obj.__class__.__name__, obj.id
                    )

        return {"id": value.pk, "text": get_str_value(self.instance) or str(value)}

    def to_internal_value(self, data):
        id = data
        text = None
        if isinstance(data, dict):
            id = data.get('id')
            text = data.get('text')

        if self.pk_field is not None:
            id = self.pk_field.to_internal_value(id)
        queryset = self.get_queryset()
        try:
            return queryset.get(pk=id)

        except ObjectDoesNotExist:
            raise ValidationError(_('Варианта "%(title)s" нет среди доступных.') % {'title': text or id})

        except (TypeError, ValueError):
            self.fail('incorrect_type', data_type=type(id).__name__)

    def get_queryset(self):
        qs = super().get_queryset()

        if self.filter_queryset:
            form_data = {}
            request = self.context.get('request', None)
            if request:
                form_data = request.data

            qs = self.filter_queryset(qs, form_data, request)

        return qs



from .char import AdminCharField
from .int import AdminIntField
from .files import Base64FileField
from .files import Base64ImageField
from .files import SVGField
from .relation import AdminPrimaryKeyRelatedField
from .choice import AdminChoiceField
from .numbers import PositiveIntegerField
from .numbers import PositiveDecimalField
from .json import AdminJsonField
from rest_framework.fields import *



from collections import OrderedDict

from django.utils.encoding import force_str
from django.utils.translation import gettext as _
from rest_framework import serializers
from rest_framework.metadata import SimpleMetadata

from custom_admin.api import fields


class CustomMetadata(SimpleMetadata):

    def get_field_info(self, field):
        """
        Given an instance of a serializer field, return a dictionary
        of metadata about it.
        """
        field_info = OrderedDict()
        field_info['type'] = self.label_lookup[field]
        field_info['required'] = getattr(field, 'required', False)
        field_info['help_text'] = False

        attrs = [
            'read_only', 'label', 'help_text',
            'min_length', 'max_length',
            'min_value', 'max_value', 'write_only', 'max_width', 'initial',
            'wysiwyg', 'multilined', 'tag_style', 'json_forms', 'list_preview',
        ]

        for attr in attrs:
            value = getattr(field, attr, None)
            if value is not None and value != '':
                field_info[attr] = value
                if not isinstance(value, (dict, list)):
                    field_info[attr] = force_str(value, strings_only=True)

        if getattr(field, 'child', None):
            field_info['child'] = self.get_field_info(field.child)
        elif getattr(field, 'fields', None):
            field_info['children'] = self.get_serializer_info(field)

        is_inst = isinstance(field, (serializers.RelatedField, serializers.ManyRelatedField))
        field_info['update_only'] = getattr(field, 'update_only', False)

        field_info['create_only'] = getattr(field, 'create_only', False)
        field_info['list_only'] = getattr(field, 'list_only', False)
        if hasattr(field, 'initial'):
            field_info['initial'] = field.initial

        if hasattr(field, 'child_relation'):
            if hasattr(field.child_relation, 'update_only'):
                field_info['update_only'] = field.child_relation.update_only
            if hasattr(field.child_relation, 'create_only'):
                field_info['create_only'] = field.child_relation.create_only

        if (not field_info.get('read_only') and not is_inst and hasattr(field, 'choices')):
            field_info['choices'] = [
                {
                    'value': choice_value,
                    'display_name': str(_(choice_name))
                }
                for choice_value, choice_name in field.choices.items()
            ]

        elif issubclass(field.__class__, (serializers.PrimaryKeyRelatedField, fields.AdminPrimaryKeyRelatedField)):
            field_info['type'] = 'primary'

            if hasattr(field, 'queryset') and field.queryset is not None:
                field_info['model_name'] = field.queryset.model.__name__
                field_info['app_label'] = field.queryset.model._meta.app_label

        elif issubclass(field.__class__, (serializers.ManyRelatedField,)):
            field_info['type'] = 'primarymany'

            if field.child_relation and field.child_relation.queryset is not None:
                field_info['model_name'] = field.child_relation.queryset.model.__name__
                field_info['app_label'] = field.child_relation.queryset.model._meta.app_label

        elif issubclass(field.__class__, (fields.Base64ImageField,)):
            field_info['type'] = 'image upload'

        elif issubclass(field.__class__, (fields.SVGField,)):
            field_info['type'] = 'svgfield'

        elif issubclass(field.__class__, (serializers.JSONField,)):
            field_info['type'] = 'json'

        filter_queryset = getattr(field, 'filter_queryset', False)
        if filter_queryset:
            field_info['filter_queryset'] = filter_queryset.__name__

        return field_info



from django.contrib.auth.models import Group
from django.utils.translation import gettext_lazy as _

from custom_admin.api.views.base_admin_viewset import BaseAdminViewSet


class GroupAdminViewSet(BaseAdminViewSet):
    title = _('Роли')
    queryset = Group.objects.all()
    search_fields = ('name', )
    list_display = (
        'id',
        'name',
    )



from collections import OrderedDict

from django.db import models
from django_filters import filters, filterset
from django_filters import rest_framework as drf_filters
from rest_framework import fields as drf_fields

from custom_admin.api.filters.date_range import AdminDateFromToRangeFilter


class AdminFilterSetMetaclass(filterset.FilterSetMetaclass):
    @classmethod
    def get_declared_filters(cls, bases, attrs):
        new_filters = [
            (filter_name, attrs.pop(filter_name))
            for filter_name, obj in list(attrs.items())
            if isinstance(obj, (filters.Filter, drf_fields.Field))
        ]

        for filter_name, f in new_filters:
            if hasattr(f, 'field_name') and getattr(f, 'field_name', None) is None:
                f.field_name = filter_name

        known = set(attrs)

        def visit(name):
            known.add(name)
            return name

        base_filters = [
            (visit(name), f)
            for base in bases if hasattr(base, 'declared_filters')
            for name, f in base.declared_filters.items() if name not in known
        ]

        return OrderedDict(base_filters + new_filters)


class BaseAdminFilterSet(drf_filters.FilterSet, metaclass=AdminFilterSetMetaclass):
    FILTER_DEFAULTS = drf_filters.FilterSet.FILTER_DEFAULTS
    FILTER_DEFAULTS[models.ForeignKey] = {
        'filter_class': filters.ModelMultipleChoiceFilter,
        'extra': lambda f: {
            'queryset': filterset.remote_queryset(f),
        }
    }
    FILTER_DEFAULTS[models.DateTimeField] = {
        'filter_class': AdminDateFromToRangeFilter,
    }



from django.urls import path
from rest_framework import routers

from custom_admin.api import views
from custom_admin.utils.register_admin_viewsets import register_admin_viewsets

admin_router = routers.SimpleRouter()

register_admin_viewsets(admin_router)

urlpatterns = [
    path('autocompete/<str:app_label>/<str:model_name>/', views.AutoCompeteView.as_view()),
    path('token-auth/', views.AdminAuthToken.as_view()),

    path(
        'get_sections/',
        views.GetAdminSectionsAPIView.as_view(router=admin_router),
        name='get-sections',
    ),
]

urlpatterns += admin_router.urls



import base64
import io
import sys
import urllib

from django.core import validators
from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import UploadedFile
from django.forms import FileField as DjangoFileField
from django.utils.translation import gettext as _
from rest_framework import fields, serializers


class Base64Serializer(serializers.Serializer):
    name = serializers.CharField(required=False)
    file = serializers.CharField(required=False)

    def validate_file(self, file):
        if not file.startswith('data:image'):
            raise ValidationError('Bad base64 format')
        return file


class Base64FileField(fields.FileField):
    list_preview: bool = False

    def __init__(self, *args, **kwargs):
        self.list_preview = kwargs.pop('list_preview', False)
        super().__init__(*args, **kwargs)

    def to_internal_value(self, data):
        serializer = Base64Serializer(data=data)
        serializer.is_valid(raise_exception=True)

        file_base64 = serializer.validated_data.get('file')
        if not file_base64:
            raise serializers.SkipField()

        content_type, base64_str = file_base64.split(",")
        upload_file = io.BytesIO(base64.b64decode(base64_str))
        file_object = UploadedFile(
            file=upload_file,
            name=serializer.validated_data['name'],
            content_type=content_type,
            size=sys.getsizeof(upload_file),
        )
        return super().to_internal_value(file_object)

    def to_representation(self, file):
        url = super().to_representation(file)
        if not url:
            return url

        request = self.context['request']
        url = urllib.parse.unquote(request.build_absolute_uri(url))
        return {
            'name': file.name,
            'url': url,
        }


def validate_image_file_extension(value):
    additional = ['svg']
    return validators.FileExtensionValidator(
        allowed_extensions=validators.get_available_image_extensions() + additional
    )(value)


class SVGDjangoFileField(DjangoFileField):
    default_validators = [validate_image_file_extension]

    def to_python(self, data):
        from PIL import Image

        f = super().to_python(data)
        if f is None:
            return None

        if hasattr(data, 'temporary_file_path'):
            file = data.temporary_file_path()
        else:
            if hasattr(data, 'read'):
                file = io.BytesIO(data.read())
            else:
                file = io.BytesIO(data['content'])

        if not data.name.endswith('.svg'):
            try:
                image = Image.open(file)
                image.verify()

                f.image = image
                f.content_type = Image.MIME.get(image.format)
            except Exception as exc:
                print(exc)
                raise ValidationError(
                    self.error_messages['invalid_image'],
                    code='invalid_image',
                ) from exc

        if hasattr(f, 'seek') and callable(f.seek):
            f.seek(0)
        return f


class Base64ImageField(Base64FileField, fields.FileField):
    default_error_messages = {
        'invalid_image': _(
            'Загрузите корректное изображение. Загруженный файл либо не является изображением, либо поврежден.'
        ),
    }

    def to_internal_value(self, data):
        file_object = super().to_internal_value(data)
        django_field = SVGDjangoFileField()
        django_field.error_messages = self.error_messages
        return django_field.clean(file_object)


class SVGField(serializers.CharField):
    ''' For raw html '''



import typing
from dataclasses import asdict, dataclass

from rest_framework import serializers

INLINE_VIEWSET_NAME_ERROR = '''
View {class_name}.related_inlines[{i}].viewset_name not existed.

Plsease, check manage.py custom_admin_viewsets_list'''

INLINE_VIEWSET_NOT_ORM = '''
View {class_name}.related_inlines[{i}].viewset_name error:

Viewset {target_viewset} don't have get_model.
Are you sure its ORM viewset?'''


@dataclass
class RelatedInline:
    title: str

    # manage.py custom-admin-viewsets-list
    viewset_name: str

    inline_slug: str

    # In case of ORM related viewset
    back_relation_name: typing.Optional[str] = None

    # https://pictogrammers.com/library/mdi/
    icon: typing.Optional[str] = None

    def asdict(self) -> dict:
        return asdict(self)

    def validate(self, viewset, index):
        from custom_admin.utils.register_admin_viewsets import _ADMIN_VIEWS
        target_viewset = _ADMIN_VIEWS[self.viewset_name]
        if not target_viewset:
            raise serializers.ValidationError(INLINE_VIEWSET_NAME_ERROR.format(class_name=viewset.__name__, i=index))

        if self.back_relation_name:
            if not hasattr(target_viewset, 'get_model'):
                raise serializers.ValidationError(INLINE_VIEWSET_NOT_ORM.format(
                    class_name=viewset.__name__,
                    i=index,
                    target_viewset=target_viewset.__name__,
                ))



import asyncio as aio
import logging
import typing

from asgiref.sync import sync_to_async
from django.db.models.fields.related_descriptors import ForwardManyToOneDescriptor, ManyToManyDescriptor
from django.utils.decorators import classonlymethod
from django.utils.translation import gettext as _
from rest_framework import mixins, serializers, viewsets
from rest_framework.pagination import PageNumberPagination
from rest_framework.parsers import JSONParser, MultiPartParser

from custom_admin.api.actions import AdminActionMixIn, export_csv_action
from custom_admin.api.backends import CustomFilterBackend, CustomOrderingFilter, CustomSearchFilter
from custom_admin.api.inline_relation import RelatedInline
from custom_admin.api.inlines import ViewActionsInlineMixIn
from custom_admin.api.permissions import AdminPermission, AdminViewPermission
from custom_admin.controllers.custom_metadata import CustomMetadata
from custom_admin.utils.async_mixin import AsyncMixin

log = logging.getLogger('admin')


class AdminPaginator(PageNumberPagination):
    page_size = 25
    page_size_query_param = 'limit'


class BaseAdminDataViewSet(ViewActionsInlineMixIn, viewsets.ViewSet):
    icon: typing.Optional[str] = None  # # https://pictogrammers.com/library/mdi/
    hide_in_navigation: typing.Optional[bool] = None

    throttle_classes = []
    parser_classes = (MultiPartParser, JSONParser)
    metadata_class = CustomMetadata
    permission_classes = (AdminPermission, AdminViewPermission)

    viewname = None
    title = None

    @classmethod
    def get_model(cls) -> typing.Optional[typing.Any]:
        return None

    @classmethod
    def get_view_viewname(cls):
        viewname = getattr(cls, 'viewname', None)
        if viewname:
            return viewname

        if hasattr(cls, 'get_model'):
            view_model = cls.get_model()
            if view_model:
                return view_model.__name__.lower()

        return cls.__name__.lower()

    @classmethod
    def get_view_title(cls):
        viewname = getattr(cls, 'title', None)
        if viewname:
            return viewname

        if hasattr(cls, 'get_model'):
            view_model = cls.get_model()
            if view_model:
                return _(view_model._meta.verbose_name_plural)

        return cls.__name__.lower()

    @classonlymethod
    def as_view(cls, *args, **initkwargs):
        # parameters of rest_framework.decorators.action
        initkwargs.pop('type', None)
        initkwargs.pop('filterset_class', None)
        view = super().as_view(*args, **initkwargs)

        async def async_view(*args, **kwargs):
            # wait for the `dispatch` method
            return await view(*args, **kwargs)
        async_view.csrf_exempt = True
        return async_view

    async def dispatch(self, request, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        request = self.initialize_request(request, *args, **kwargs)
        self.request = request
        self.headers = self.default_response_headers  # deprecate?

        try:
            # MODIFIED HERE
            await sync_to_async(self.initial)(
                request, *args, **kwargs
            )

            if request.method.lower() in self.http_method_names:
                handler = getattr(
                    self, request.method.lower(),
                    self.http_method_not_allowed
                )
            else:
                handler = self.http_method_not_allowed

            handler_kwargs = getattr(handler, 'kwargs', {})
            handler_inline_type = handler_kwargs.get('inline_type')
            if handler_inline_type is not None:

                async_handle_inline = self.handle_inline
                if not aio.iscoroutinefunction(async_handle_inline):
                    async_handle_inline = sync_to_async(async_handle_inline)

                response = await async_handle_inline(handler, request, handler_kwargs, *args, **kwargs)
            else:
                if not aio.iscoroutinefunction(handler):
                    handler = sync_to_async(handler)
                response = await handler(request, *args, **kwargs)

        except Exception as exc:
            response = self.handle_exception(exc)

        self.response = self.finalize_response(request, response, *args, **kwargs)
        return self.response


class BaseAdmin(AdminActionMixIn, BaseAdminDataViewSet, AsyncMixin):
    '''
    serializer_class is optional, will be used all fields by default or
    you can set serializer_fields
    '''
    pagination_class = AdminPaginator
    filter_backends = [
        CustomSearchFilter, CustomOrderingFilter, CustomFilterBackend
    ]
    search_fields = ['id']
    ordering_fields = ['id']
    ordering = ['-id']
    filterset_fields = []

    fixed_columns = ('id',)

    # Set None for diplay all fields
    filds_list = ['id']

    related_inlines: typing.List[RelatedInline] = []

    @classmethod
    def get_related_inlines(cls) -> typing.List[RelatedInline]:
        result = []
        for i, inline in enumerate(cls.related_inlines):

            if not isinstance(inline, RelatedInline):
                raise serializers.ValidationError(f'View {cls.__name__}.related_inlines[{i}] error: is not an instance of RelatedInline')

            inline.validate(cls, i)
            result.append(inline)

        return result

    @classmethod
    def get_model(cls) -> typing.Optional[typing.Any]:
        return cls.get_serializer_class().Meta.model

    def get_queryset(self):
        qs = super().get_queryset()

        relfilterid = self.request.query_params.get('relfilterid')
        relfilter = self.request.query_params.get('relfilter')
        if relfilterid and relfilter:
            qs = qs.filter(**{relfilter: relfilterid})

        return qs

    def get_serializer_context(self):
        '''
        Если передаются query_params c id объекта, то добавляем его в контекст сериализатора
        '''
        ctx = {
            'request': self.request,
            'format': self.format_kwarg,
            'view': self
        }
        relfilterid = self.request.query_params.get('relfilterid')
        relfilter = self.request.query_params.get('relfilter')
        if relfilterid and relfilter:
            relobject = None
            try:
                rel_model_descriptor = getattr(self.get_model(), relfilter)
                if isinstance(rel_model_descriptor, ManyToManyDescriptor):
                    relobject = rel_model_descriptor.rel.related_model.objects.get(id=relfilterid)
                if isinstance(rel_model_descriptor, ForwardManyToOneDescriptor):
                    relobject = rel_model_descriptor.field.related_model.objects.get(id=relfilterid)
                ctx.update({'relobject': relobject})
            except Exception:
                pass

        ctx.update({'viewname': self.get_view_viewname()})
        return ctx

    @classmethod
    def get_serializer_class(cls):
        if hasattr(cls, 'serializer_class'):
            serializer_class = getattr(cls, 'serializer_class')
            if serializer_class:
                return serializer_class

        from custom_admin.api.serializers import AdminModelSerializer

        class GenericAdminSerializer(AdminModelSerializer):
            class Meta:
                queryset = getattr(cls, 'queryset', None)
                if queryset is None:
                    raise Exception(f'{cls.__name__}: To create a generic serializer, you need to specify a queryset')

                model = queryset.model
                fields = getattr(cls, 'serializer_fields', '__all__')
                ref_name = model.__name__

        cls.serializer_class = GenericAdminSerializer
        return cls.serializer_class


class WithoutCreateBaseAdminViewSet(
        mixins.RetrieveModelMixin,
        mixins.UpdateModelMixin,
        mixins.ListModelMixin,
        BaseAdmin,
        viewsets.GenericViewSet,

):
    pass


class WithoutUpdateBaseAdminViewSet(
        mixins.RetrieveModelMixin,
        mixins.CreateModelMixin,
        mixins.ListModelMixin,
        BaseAdmin,
        viewsets.GenericViewSet,
):
    pass


class BaseAdminViewSet(mixins.CreateModelMixin, WithoutCreateBaseAdminViewSet):
    pass


class ReadOnlyBaseAdminViewSet(
        mixins.RetrieveModelMixin,
        mixins.ListModelMixin,
        BaseAdmin,
        viewsets.GenericViewSet,
):
    actions = [export_csv_action]


class ListBaseAdminViewSet(
        mixins.ListModelMixin,
        BaseAdmin,
        viewsets.GenericViewSet,
):
    actions = [export_csv_action]



from django.contrib.auth import get_user_model

from custom_admin.api.serializers.base_serializer import AdminModelSerializer


class ProfileSerializer(AdminModelSerializer):
    class Meta:
        model = get_user_model()
        fields = [
            'pk',
            'username',
        ]



import logging

from custom_admin.controllers.permissions import CheckPermissions, PermissioinType
from rest_framework.permissions import BasePermission

logger = logging.getLogger('custom_admin')


class AdminPermission(BasePermission):
    def has_permission(self, request, view):
        user = request.user

        if not user.is_staff or not user.is_active:
            logger.info(
                'ADMIN BYPASS User #%s username:%s is_staff:%s is_active:%s',
                user.id, user.username, user.is_staff, user.is_active,
            )
            return False

        return True


class AdminViewPermission(BasePermission):
    def has_permission(self, request, view):
        return CheckPermissions(request.user).has_action_perm(view, view.action)



import asyncio as aio
import logging

from asgiref.sync import sync_to_async
from django.http import HttpResponse
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.response import Response

from custom_admin.api.actions.delete_action import delete_action
from custom_admin.api.actions.export_csv_action import export_csv_action
from custom_admin.controllers import AdminLogManager

log = logging.getLogger('admin')


class ActionSerializer(serializers.Serializer):
    action = serializers.CharField(required=True)
    ids = serializers.ListField(child=serializers.CharField(), required=False)
    send_to_all = serializers.BooleanField(default=False)
    form_data = serializers.JSONField(required=False, allow_null=True)


class AdminActionMixIn:

    actions = [
        delete_action, export_csv_action,
    ]

    @action(detail=False, methods=['post'])
    async def send_action(self, request, pk=None):
        serializer = ActionSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        action_name = serializer.validated_data['action']
        ids = serializer.validated_data['ids']
        send_to_all = serializer.validated_data['send_to_all']
        form_data = serializer.validated_data['form_data']

        actions_dict = {
            action.__name__: action
            for action in self.actions
        }

        if action_name not in actions_dict:
            return Response(f'View {self.get_view_viewname()} doesnt have action "{action_name}"', status=400)

        qs = await sync_to_async(self.get_queryset)()
        qs = await sync_to_async(self.filter_queryset)(qs)

        if not send_to_all:
            qs = qs.filter(id__in=ids)

        action_fn = actions_dict[action_name]
        if not aio.iscoroutinefunction(action_fn):
            action_fn = sync_to_async(action_fn)

        actions_result = await action_fn(self, request, qs, form_data)

        if isinstance(actions_result, (tuple, list)):
            action_messages, code = actions_result

            if isinstance(action_messages, (str)) or action_messages.__class__.__name__ == '__proxy__':
                action_messages = [action_messages]

            action_request = Response({'action_messages': action_messages}, status=code)

        elif isinstance(actions_result, (Response, HttpResponse)):
            action_request = actions_result

        await sync_to_async(self.log_action)(action_name, actions_dict[action_name].short_description, request)
        return action_request

    def log_action(self, action_name: str, short_description: str, request):
        AdminLogManager(self.request.user).register_action(
            action_name, short_description, self, request
        )
